@@ -27,10 +27,6 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
-#ifdef HAVE_CLUTTER
-#include <clutter/clutter.h>
-#endif
-
 #include "ar-debug.h"
 #include "ar-runtime.h"
 #include "ar-string-utils.h"
@@ -50,7 +46,7 @@ struct _AisleriotGameClass
 };
 
 enum {
-  START_GAME_LAMBDA,
+  NEW_GAME_LAMBDA,
   BUTTON_PRESSED_LAMBDA,
   BUTTON_RELEASED_LAMBDA,
   BUTTON_CLICKED_LAMBDA,
@@ -67,13 +63,29 @@ enum {
   LAST_MANDATORY_LAMBDA = TIMEOUT_LAMBDA
 };
 
+static const char lambda_names[] = {
+  "new-game\0"
+  "button-pressed\0"
+  "button-released\0"
+  "button-clicked\0"
+  "button-double-clicked\0"
+  "game-over\0"
+  "winning-game\0"
+  "hint\0"
+  "get-options\0"
+  "apply-options\0"
+  "timeout\0"
+  "droppable\0"
+  "dealable\0"
+};
+
 struct _AisleriotGame
 {
   GObject parent_instance;
 
   GPtrArray *slots;
 
-  char *game_file;
+  char *game_module;
 
   GRand *rand;
   GRand *saved_rand;
@@ -83,7 +95,7 @@ struct _AisleriotGame
   GTimer *timer;
 
   int timeout;
-  int score;
+  char *score;
 
   double width;
   double height;
@@ -161,17 +173,6 @@ set_game_state (AisleriotGame *game,
   }
 }
 
-static void
-set_game_score (AisleriotGame *game,
-                int score)
-{
-  if (score != game->score) {
-    game->score = score;
-
-    g_object_notify (G_OBJECT (game), "score");
-  }
-}
-
 static void
 set_game_undoable (AisleriotGame *game,
                    gboolean enabled)
@@ -215,7 +216,7 @@ update_statistics (AisleriotGame *game)
   AisleriotStatistic current_stats;
   time_t t;
 
-  aisleriot_conf_get_statistic (game->game_file, &current_stats);
+  aisleriot_conf_get_statistic (game->game_module, &current_stats);
 
   current_stats.total++;
 
@@ -233,7 +234,7 @@ update_statistics (AisleriotGame *game)
     }
   }
 
-  aisleriot_conf_set_statistic (game->game_file, &current_stats);
+  aisleriot_conf_set_statistic (game->game_module, &current_stats);
 }
 
 static void
@@ -246,16 +247,7 @@ clear_slots (AisleriotGame *game,
   for (i = 0; i < n_slots; ++i) {
     ArSlot *slot = game->slots->pdata[i];
 
-#ifdef HAVE_CLUTTER
-    if (slot->slot_renderer) {
-      clutter_actor_destroy (slot->slot_renderer);
-      g_object_unref (slot->slot_renderer);
-    }
-
-    g_byte_array_free (slot->old_cards, TRUE);
-#else
     g_ptr_array_free (slot->card_images, TRUE);
-#endif /* HAVE_CLUTTER */
     g_byte_array_free (slot->cards, TRUE);
 
     g_slice_free (ArSlot, slot);
@@ -309,7 +301,7 @@ cscmi_exception_get_backtrace (SCM tag, SCM throw_args)
 
   message = g_string_sized_new (1024);
 
-  g_string_append_printf (message, "Variation: %s\n", aisleriot_game_get_game_file (game));
+  g_string_append_printf (message, "Variation: %s\n", aisleriot_game_get_game_module (game));
 #if 0
   g_string_append_printf (message, "Seed: %u\n", game->seed);
 #endif
@@ -318,7 +310,7 @@ cscmi_exception_get_backtrace (SCM tag, SCM throw_args)
 
   port = scm_open_output_string ();
   scm_display (throw_args, port);
-  string = scm_to_locale_string (scm_get_output_string (port));
+  string = scm_to_utf8_string (scm_get_output_string (port));
   scm_dynwind_free (string);
   scm_close_output_port (port);
   g_string_append (message, string);
@@ -326,7 +318,7 @@ cscmi_exception_get_backtrace (SCM tag, SCM throw_args)
   port = scm_open_output_string ();
   g_string_append (message, "\nScheme tag:\n\t");
   scm_display (tag, port);
-  string = scm_to_locale_string (scm_get_output_string (port));
+  string = scm_to_utf8_string (scm_get_output_string (port));
   scm_dynwind_free (string);
   scm_close_output_port (port);
   g_string_append (message, string);
@@ -336,7 +328,7 @@ cscmi_exception_get_backtrace (SCM tag, SCM throw_args)
   if (!scm_is_false (stack)) {
     port = scm_open_output_string ();
     scm_display_backtrace (stack, port, SCM_UNDEFINED, SCM_UNDEFINED);
-    string = scm_to_locale_string (scm_get_output_string (port));
+    string = scm_to_utf8_string (scm_get_output_string (port));
     scm_dynwind_free (string);
     scm_close_output_port (port);
     g_string_append (message, string);
@@ -598,7 +590,7 @@ cscmi_add_slot (SCM slot_data)
 
   if (game->state > GAME_BEGIN) {
     return scm_throw (scm_from_locale_symbol ("aisleriot-invalid-call"),
-                      scm_list_1 (scm_from_locale_string ("Cannot add a new slot after the game has started.")));
+                      scm_list_1 (scm_from_utf8_string ("Cannot add a new slot after the game has started.")));
   }
 
 #define EQUALS_SYMBOL(string,object) (scm_is_true (scm_equal_p (scm_from_locale_symbol (string), object)))
@@ -621,7 +613,9 @@ cscmi_add_slot (SCM slot_data)
   /* 3rd argument is the slot type (optionally) */
   slot_type = SCM_CDDDR (slot_data);
   if (slot_type != SCM_EOL) {
-    if (EQUALS_SYMBOL ("foundation", SCM_CAR (slot_type))) {
+    if (EQUALS_SYMBOL ("chooser", SCM_CAR (slot_type))) {
+      type = AR_SLOT_CHOOSER;
+    } else if (EQUALS_SYMBOL ("foundation", SCM_CAR (slot_type))) {
       type = AR_SLOT_FOUNDATION;
     } else if (EQUALS_SYMBOL ("reserve", SCM_CAR (slot_type))) {
       type = AR_SLOT_RESERVE;
@@ -638,7 +632,7 @@ cscmi_add_slot (SCM slot_data)
 
 #ifdef GNOME_ENABLE_DEBUG
   _AR_DEBUG_IF (AR_DEBUG_SCHEME) {
-    static const char *types[] = { "unknown", "foundation", "reserve", "stock", "tableau", "waste" };
+    static const char *types[] = { "unknown", "chooser", "foundation", "reserve", "stock", "tableau", "waste" };
 
     ar_debug_print (AR_DEBUG_SCHEME,
                         "Adding new slot %d type %s\n",
@@ -665,11 +659,7 @@ cscmi_add_slot (SCM slot_data)
   slot->expanded_down = expanded_down != FALSE;
   slot->expanded_right = expanded_right != FALSE;
 
-#ifdef HAVE_CLUTTER
-  slot->old_cards = g_byte_array_sized_new (SLOT_CARDS_N_PREALLOC);
-#else
   slot->card_images = g_ptr_array_sized_new (SLOT_CARDS_N_PREALLOC);
-#endif
 
   slot->needs_update = TRUE;
 
@@ -680,36 +670,6 @@ cscmi_add_slot (SCM slot_data)
 }
 
 /* Scheme functions */
-static SCM
-scm_gettext (SCM message)
-{
-  char *input;
-  const char *output;
-  SCM translated = SCM_UNDEFINED;
-
-  if (!scm_is_string (message))
-    return message;
-
-  scm_dynwind_begin (0);
-
-  input = scm_to_locale_string (message);
-  scm_dynwind_free (input);
-  if (!input)
-    goto out;
-
-  output = g_dgettext (NULL, input);
-
-  if (input != output) {
-    translated = scm_from_locale_string (output);
-  } else {
-    translated = message;
-  }
-
-out:
-  scm_dynwind_end ();
-
-  return translated;
-}
 
 static SCM
 scm_undo_set_sensitive (SCM in_state)
@@ -780,7 +740,7 @@ scm_set_statusbar_message (SCM message)
 
   scm_dynwind_begin (0);
 
-  str = scm_to_locale_string (message);
+  str = scm_to_utf8_string (message);
   scm_dynwind_free (str);
   if (!str)
     goto out;
@@ -893,7 +853,7 @@ scm_set_lambda (SCM start_game_lambda,
 {
   AisleriotGame *game = app_game;
 
-  game->lambdas[START_GAME_LAMBDA] = start_game_lambda;
+  game->lambdas[NEW_GAME_LAMBDA] = start_game_lambda;
   game->lambdas[BUTTON_PRESSED_LAMBDA] = pressed_lambda;
   game->lambdas[BUTTON_RELEASED_LAMBDA] = released_lambda;
   game->lambdas[BUTTON_CLICKED_LAMBDA] = clicked_lambda;
@@ -928,6 +888,28 @@ scm_set_lambda (SCM start_game_lambda,
   return SCM_EOL;
 }
 
+static SCM
+scm_set_lambda_x (SCM symbol,
+                  SCM lambda)
+{
+  AisleriotGame *game = app_game;
+  const char *lambda_name;
+  int i;
+
+  lambda_name = lambda_names;
+  for (i = 0; i < N_LAMBDAS; ++i) {
+    if (scm_is_true (scm_equal_p (symbol, scm_from_locale_symbol (lambda_name)))) {
+      game->lambdas[i] = lambda;
+      return SCM_EOL;
+    }
+
+    lambda_name += strlen (lambda_name) + 1;
+  }
+
+  return scm_throw (scm_from_locale_symbol ("aisleriot-invalid-call"),
+                    scm_list_1 (scm_from_utf8_string ("Unknown lambda name in set-lambda!")));
+}
+
 static SCM
 scm_myrandom (SCM range)
 {
@@ -948,33 +930,24 @@ scm_click_to_move_p (void)
 }
 
 static SCM
-scm_get_score (void)
+scm_update_score (SCM new_score)
 {
   AisleriotGame *game = app_game;
+  char *score;
 
-  return scm_from_int (game->score);
-}
+  score = scm_to_utf8_string (new_score);
+  if (g_strcmp0 (score, game->score) != 0) {
+    free (game->score);
+    game->score = score;
 
-static SCM
-scm_set_score (SCM new_score)
-{
-  AisleriotGame *game = app_game;
+    g_object_notify (G_OBJECT (game), "score");
+  } else {
+    free (score);
+  }
 
-  set_game_score (game, scm_to_int (new_score));
   return new_score;
 }
 
-static SCM
-scm_add_to_score (SCM delta)
-{
-  AisleriotGame *game = app_game;
-  int new_score;
-
-  new_score = game->score + scm_to_int (delta);
-  set_game_score (game, new_score);
-  return scm_from_int (new_score);
-}
-
 static SCM
 scm_set_timeout (SCM new)
 {
@@ -1034,7 +1007,7 @@ scm_delayed_call (SCM callback)
   /* We can only have one pending delayed call! */
   if (game->delayed_call_timeout_id != 0) {
     return scm_throw (scm_from_locale_symbol ("aisleriot-invalid-call"),
-                      scm_list_1 (scm_from_locale_string ("Already have a delayed callback pending.")));
+                      scm_list_1 (scm_from_utf8_string ("Already have a delayed callback pending.")));
   }
 
   /* We need to protect the callback data from being GC'd until the
@@ -1052,12 +1025,12 @@ scm_delayed_call (SCM callback)
 }
 
 static void
-cscm_init (void)
+cscm_init (void *data G_GNUC_UNUSED)
 {
   /* Let the scheme side of things know about our C functions. */
   scm_c_define_gsubr ("set-feature-word!", 1, 0, 0, scm_set_feature_word);
   scm_c_define_gsubr ("get-feature-word", 0, 0, 0, scm_get_feature_word);
-  scm_c_define_gsubr ("set-statusbar-message", 1, 0, 0,
+  scm_c_define_gsubr ("set-statusbar-message-c", 1, 0, 0,
                       scm_set_statusbar_message);
   scm_c_define_gsubr ("reset-surface", 0, 0, 0, scm_reset_surface);
   scm_c_define_gsubr ("add-slot", 1, 0, 0, cscmi_add_slot);
@@ -1068,18 +1041,38 @@ cscm_init (void)
   scm_c_define_gsubr ("set-slot-x-expansion!", 2, 0, 0,
                       scm_set_slot_x_expansion);
   scm_c_define_gsubr ("set-lambda", 8, 0, 1, scm_set_lambda);
-  scm_c_define_gsubr ("random", 1, 0, 0, scm_myrandom);
+  scm_c_define_gsubr ("set-lambda!", 2, 0, 0, scm_set_lambda_x);
+  scm_c_define_gsubr ("aisleriot-random", 1, 0, 0, scm_myrandom);
   scm_c_define_gsubr ("click-to-move?", 0, 0, 0, scm_click_to_move_p);
-  scm_c_define_gsubr ("get-score", 0, 0, 0, scm_get_score);
-  scm_c_define_gsubr ("set-score!", 1, 0, 0, scm_set_score);
+  scm_c_define_gsubr ("update-score", 1, 0, 0, scm_update_score);
   scm_c_define_gsubr ("get-timeout", 0, 0, 0, scm_get_timeout);
   scm_c_define_gsubr ("set-timeout!", 1, 0, 0, scm_set_timeout);
   scm_c_define_gsubr ("delayed-call", 1, 0, 0, scm_delayed_call);
-  scm_c_define_gsubr ("add-to-score!", 1, 0, 0, scm_add_to_score);
-  scm_c_define_gsubr ("_", 1, 0, 0, scm_gettext);
   scm_c_define_gsubr ("undo-set-sensitive", 1, 0, 0, scm_undo_set_sensitive);
   scm_c_define_gsubr ("redo-set-sensitive", 1, 0, 0, scm_redo_set_sensitive);
   scm_c_define_gsubr ("dealable-set-sensitive", 1, 0, 0, scm_dealable_set_sensitive);
+
+  scm_c_export ("set-feature-word!", 
+                "get-feature-word", 
+                "set-statusbar-message-c",
+                "reset-surface",
+                "add-slot", 
+                "get-slot", 
+                "set-cards-c!",
+                "set-slot-y-expansion!", 
+                "set-slot-x-expansion!",
+                "set-lambda", 
+                "set-lambda!",
+                "aisleriot-random", 
+                "click-to-move?", 
+                "update-score", 
+                "get-timeout", 
+                "set-timeout!", 
+                "delayed-call", 
+                "undo-set-sensitive", 
+                "redo-set-sensitive", 
+                "dealable-set-sensitive",
+                NULL);
 }
 
 static void
@@ -1134,6 +1127,7 @@ aisleriot_game_init (AisleriotGame *game)
   game->timer = g_timer_new ();
 
   game->timeout = 60 * 60;
+  game->score = NULL;
 }
 
 static GObject *
@@ -1168,7 +1162,7 @@ aisleriot_game_finalize (GObject *object)
   clear_slots (game, FALSE);
   g_ptr_array_free (game->slots, TRUE);
 
-  g_free (game->game_file);
+  g_free (game->game_module);
 
   g_timer_destroy (game->timer);
 
@@ -1177,6 +1171,8 @@ aisleriot_game_finalize (GObject *object)
   if (game->saved_rand)
     g_rand_free (game->saved_rand);
 
+  free (game->score);
+
   app_game = NULL;
 
   G_OBJECT_CLASS (aisleriot_game_parent_class)->finalize (object);
@@ -1201,10 +1197,10 @@ aisleriot_game_get_property (GObject *object,
       g_value_set_boolean (value, game->can_deal);
       break;
     case PROP_GAME_FILE:
-      g_value_set_string (value, game->game_file);
+      g_value_set_string (value, game->game_module);
       break;
     case PROP_SCORE:
-      g_value_set_int (value, game->score);
+      g_value_set_string (value, game->score);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1219,6 +1215,8 @@ aisleriot_game_class_init (AisleriotGameClass *klass)
   GType param_types[] = { G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE };
   GType error_types[] = { G_TYPE_ERROR | G_SIGNAL_TYPE_STATIC_SCOPE };
   GType ptr_types[] = { G_TYPE_POINTER };
+  SCM variable;
+  char *path;
 
   gobject_class->constructor = aisleriot_game_constructor;
   gobject_class->finalize = aisleriot_game_finalize;
@@ -1319,10 +1317,10 @@ aisleriot_game_class_init (AisleriotGameClass *klass)
   g_object_class_install_property
     (gobject_class,
      PROP_SCORE,
-     g_param_spec_int ("score", NULL, NULL,
-                       G_MININT, G_MAXINT, 0,
-                       G_PARAM_READABLE |
-                       G_PARAM_STATIC_STRINGS));
+     g_param_spec_string ("score", NULL, NULL,
+                          NULL,
+                          G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
     (gobject_class,
@@ -1332,8 +1330,26 @@ aisleriot_game_class_init (AisleriotGameClass *klass)
                         G_PARAM_READABLE |
                         G_PARAM_STATIC_STRINGS));
 
-  /* Initialise our scheme interfaces */
-  cscm_init ();
+  scm_c_define_module ("aisleriot interface", cscm_init, NULL);
+
+  /* Append load-path */
+  path = g_build_filename (ar_runtime_get_directory (AR_RUNTIME_PKG_DATA_DIRECTORY),
+                           "guile",
+                           SCM_EFFECTIVE_VERSION,
+                           NULL);
+  variable = scm_c_module_lookup (scm_the_root_module (), "%load-path");
+  scm_variable_set_x (variable, scm_append_x (scm_list_2 (scm_variable_ref (variable),
+                                                          scm_list_1 (scm_from_utf8_string (path)))));
+  g_free (path);
+
+  path = g_build_filename (ar_runtime_get_directory (AR_RUNTIME_PKG_LIBRARY_DIRECTORY),
+                           "guile",
+                           SCM_EFFECTIVE_VERSION,
+                           NULL);
+  variable = scm_c_module_lookup (scm_the_root_module (), "%load-compiled-path");
+  scm_variable_set_x (variable, scm_append_x (scm_list_2 (scm_variable_ref (variable),
+                                                          scm_list_1 (scm_from_utf8_string (path)))));
+  g_free (path);
 }
 
 /* public API */
@@ -1369,6 +1385,10 @@ ar_slot_get_type_string (ArSlot *slot)
     case AR_SLOT_UNKNOWN:
       text = NULL;
       break;
+    case AR_SLOT_CHOOSER:
+      /* Translators: this is the name of a type of card slot */
+      text = C_("slot type", "chooser");
+      break;
     case AR_SLOT_FOUNDATION:
       /* Translators: this is the name of a type of card slot */
       text = C_("slot type", "foundation");
@@ -1417,6 +1437,10 @@ ar_slot_get_hint_string (ArSlot *slot,
   card_name = ar_card_get_locale_name (CARD (slot->cards->data[cardid]));
 
   switch (slot->type) {
+    case AR_SLOT_CHOOSER:
+      /* Translators: %s is the name of the card; "chooser" is the name of a type of card slot */
+      return g_strdup_printf (C_("slot hint", "%s on chooser"), card_name);
+
     case AR_SLOT_FOUNDATION:
       /* Translators: %s is the name of the card; "foundation" is the name of a type of card slot */
       return g_strdup_printf (C_("slot hint", "%s on foundation"), card_name);
@@ -1614,27 +1638,17 @@ static SCM
 game_scm_load_game (void *user_data)
 {
   AisleriotGame *game = app_game;
-  const char *game_file = user_data;
-  char *path;
+  const char *game_module = user_data;
   int i;
 
   scm_dynwind_begin (0);
 
-  /* Although this line slows down game switching by a noticeable amount, we
-    * add it here in order to make sure all the original functions are
-    * "clean". */
-  path = ar_runtime_get_file (AR_RUNTIME_GAMES_DIRECTORY, "sol.scm");
-  scm_dynwind_unwind_handler (g_free, path, SCM_F_WIND_EXPLICITLY);
-  scm_c_primitive_load (path);
-
-  path = ar_runtime_get_file (AR_RUNTIME_GAMES_DIRECTORY, game_file);
-  scm_dynwind_unwind_handler (g_free, path, SCM_F_WIND_EXPLICITLY);
-  scm_c_primitive_load (path);
+  scm_primitive_load_path (scm_from_utf8_string (game_module));
 
   for (i = 0; i <= LAST_MANDATORY_LAMBDA; ++i) {
     if (scm_is_false (scm_procedure_p (game->lambdas[i]))) {
       scm_throw (scm_from_locale_symbol ("aisleriot-invalid-lambda"),
-                 scm_list_3 (scm_from_locale_string ("Not a procedure"),
+                 scm_list_3 (scm_from_utf8_string ("Not a procedure"),
                              scm_from_int (i),
                              game->lambdas[i]));
     }
@@ -1642,13 +1656,13 @@ game_scm_load_game (void *user_data)
   if ((game->features & FEATURE_DROPPABLE) &&
       scm_is_false (scm_procedure_p (game->lambdas[DROPPABLE_LAMBDA])))
     scm_throw (scm_from_locale_symbol ("aisleriot-invalid-lambda"),
-               scm_list_3 (scm_from_locale_string ("Not a procedure"),
+               scm_list_3 (scm_from_utf8_string ("Not a procedure"),
                            scm_from_int (DROPPABLE_LAMBDA),
                            game->lambdas[i]));
   if ((game->features & FEATURE_DEALABLE) &&
-      scm_is_false (game->lambdas[DEALABLE_LAMBDA]))
+      scm_is_false (scm_procedure_p (game->lambdas[DEALABLE_LAMBDA])))
     scm_throw (scm_from_locale_symbol ("aisleriot-invalid-lambda"),
-               scm_list_3 (scm_from_locale_string ("Not a procedure"),
+               scm_list_3 (scm_from_utf8_string ("Not a procedure"),
                            scm_from_int (DEALABLE_LAMBDA),
                            game->lambdas[i]));
 
@@ -1660,33 +1674,33 @@ game_scm_load_game (void *user_data)
 /**
  * aisleriot_game_load_game:
  * @game:
- * @game_file: the game file to load
+ * @game_module: the game module to load
  * @error: a location for a #GError
  *
- * Loads the game @game_file into @game. If there is an error,
+ * Loads the game @game_module into @game. If there is an error,
  * @error is filled in and %FALSE returned.
  *
  * Returns: %TRUE iff loading the game succeeded
  */
 gboolean
 aisleriot_game_load_game (AisleriotGame *game,
-                          const char *game_file,
+                          const char *game_module,
                           GError **error)
 {
   GObject *object = G_OBJECT (game);
   GError *err = NULL;
   int i;
 
-  g_return_val_if_fail (game_file != NULL && game_file[0] != '\0', FALSE);
+  g_return_val_if_fail (game_module != NULL && game_module[0] != '\0', FALSE);
 
   /* FIXMEchpe: allow re-loading */
-  if (g_strcmp0 (game_file, game->game_file) == 0)
+  if (g_strcmp0 (game_module, game->game_module) == 0)
     return TRUE;
 
-  /* We use @game_file as a filename, but since it'll be used in configuration
+  /* We use @game_module as a filename, but since it'll be used in configuration
    * as a key, we also require it to be valid UTF-8 !
    */
-  if (!g_utf8_validate (game_file, -1, NULL)) {
+  if (!g_utf8_validate (game_module, -1, NULL)) {
     g_set_error (error, AISLERIOT_GAME_ERROR, GAME_ERROR_GENERIC,
                  "Invalid UTF-8");
     return FALSE;
@@ -1703,7 +1717,6 @@ aisleriot_game_load_game (AisleriotGame *game,
 
   clear_delayed_call (game);
   set_game_state (game, GAME_UNINITIALISED);
-  set_game_score (game, 0);
   set_game_undoable (game, FALSE);
   set_game_redoable (game, FALSE);
   set_game_dealable (game, FALSE);
@@ -1711,9 +1724,11 @@ aisleriot_game_load_game (AisleriotGame *game,
   game->had_exception = FALSE;
   for (i = 0; i < N_LAMBDAS; ++i)
     game->lambdas[i] = SCM_UNDEFINED;
+  g_free (game->game_module);
+  game->game_module = g_strdup (game_module);
 
   scm_c_catch (SCM_BOOL_T,
-               (scm_t_catch_body) game_scm_load_game, (void *) game_file,
+               (scm_t_catch_body) game_scm_load_game, (void *) game_module,
                game_scm_catch_handler, NULL,
                game_scm_pre_unwind_handler, &err);
 
@@ -1723,9 +1738,6 @@ aisleriot_game_load_game (AisleriotGame *game,
     return FALSE;
   }
 
-  g_free (game->game_file);
-  game->game_file = g_strdup (game_file);
-
   set_game_state (game, GAME_LOADED);
 
   g_object_notify (object, "game-file");
@@ -1756,7 +1768,7 @@ game_scm_new_game (void *user_data)
       g_rand_free (game->saved_rand);
     game->saved_rand = g_rand_copy (game->rand);
 
-    size = scm_call_0 (game->lambdas[START_GAME_LAMBDA]);
+    size = scm_call_0 (game->lambdas[NEW_GAME_LAMBDA]);
     game->width = scm_to_double (SCM_CAR (size));
     game->height = scm_to_double (SCM_CADR (size));
     scm_remember_upto_here_1 (size);
@@ -1836,7 +1848,6 @@ aisleriot_game_new_game_internal (AisleriotGame *game,
   clear_delayed_call (game);
   /* The game isn't actually in progress until the user makes a move */
   set_game_state (game, GAME_BEGIN);
-  set_game_score (game, 0);
   set_game_undoable (game, FALSE);
   set_game_redoable (game, FALSE);
 
@@ -1906,7 +1917,7 @@ aisleriot_game_restart_game (AisleriotGame *game)
 }
 
 /**
- * aisleriot_game_get_game_file:
+ * aisleriot_game_get_game_module:
  * @game:
  *
  * Returns the game filename for the currently loaded game type.
@@ -1914,9 +1925,9 @@ aisleriot_game_restart_game (AisleriotGame *game)
  * Returns: a string owned by @game; you must not modify or free it
  */
 const char *
-aisleriot_game_get_game_file (AisleriotGame *game)
+aisleriot_game_get_game_module (AisleriotGame *game)
 {
-  return game->game_file;
+  return game->game_module;
 }
 
 /**
@@ -1931,11 +1942,11 @@ aisleriot_game_get_game_file (AisleriotGame *game)
 char *
 aisleriot_game_get_name (AisleriotGame *game)
 {
-  return ar_filename_to_display_name (game->game_file);
+  return ar_filename_to_display_name (game->game_module);
 }
 
 /**
- * aisleriot_game_get_game_file:
+ * aisleriot_game_get_game_module:
  * @game:
  * @width: a location to store the width in
  * @height: a location to store the height in
@@ -2124,7 +2135,7 @@ aisleriot_game_get_hint (AisleriotGame *game)
       if (!scm_is_string (string1))
         break;
 
-      str1 = scm_to_locale_string (string1);
+      str1 = scm_to_utf8_string (string1);
       scm_dynwind_free (str1);
       if (!str1)
         break;
@@ -2139,12 +2150,12 @@ aisleriot_game_get_hint (AisleriotGame *game)
       if (!scm_is_string (string1) || !scm_is_string (string2))
         break;
 
-      str1 = scm_to_locale_string (string1);
+      str1 = scm_to_utf8_string (string1);
       scm_dynwind_free (str1);
       if (!str1)
         break;
 
-      str2 = scm_to_locale_string (string2);
+      str2 = scm_to_utf8_string (string2);
       scm_dynwind_free (str2);
       if (!str2)
         break;
@@ -2164,11 +2175,11 @@ aisleriot_game_get_hint (AisleriotGame *game)
       if (!scm_is_string (string1) || !scm_is_string (string2))
         break;
 
-      str1 = scm_to_locale_string (string1);
+      str1 = scm_to_utf8_string (string1);
       scm_dynwind_free (str1);
       if (!str1)
         break;
-      str2 = scm_to_locale_string (string2);
+      str2 = scm_to_utf8_string (string2);
       scm_dynwind_free (str2);
       if (!str2)
         break;
@@ -2184,7 +2195,7 @@ aisleriot_game_get_hint (AisleriotGame *game)
                  "Please file a bug at http://bugzilla.gnome.org "
                  "including this message and the name of the game "
                  "you were playing, which is %s.\n",
-                 aisleriot_game_get_game_file (game));
+                 aisleriot_game_get_game_module (game));
       break;
 
     case 4: /* This is deprecated (due to i18n issues) do not use. */
@@ -2192,7 +2203,7 @@ aisleriot_game_get_hint (AisleriotGame *game)
                  "Please file a bug at http://bugzilla.gnome.org "
                  "including this message and the name of the game "
                  "you were playing, which is %s.\n",
-                 aisleriot_game_get_game_file (game));
+                 aisleriot_game_get_game_module (game));
       break;
 
     default:
@@ -2264,7 +2275,7 @@ aisleriot_game_get_options (AisleriotGame *game)
       if (!scm_is_string (entryname))
         continue; /* Shouldn't happen */
 
-      entrynamestr = scm_to_locale_string (entryname);
+      entrynamestr = scm_to_utf8_string (entryname);
       scm_dynwind_free (entrynamestr);
       if (!entrynamestr)
         continue;
@@ -2508,40 +2519,90 @@ aisleriot_game_deal_cards (AisleriotGame *game)
   aisleriot_game_test_end_of_game (game);
 }
 
-#ifdef HAVE_CLUTTER
-
-void
-aisleriot_game_get_card_offset (ArSlot *slot,
-                                guint card_num,
-                                gboolean old_cards,
-                                gint *xoff, gint *yoff)
+/**
+ * aisleriot_game_get_score:
+ * @game:
+ * 
+ * Returns: (transfer none): the current score as a string
+ */
+const char *
+aisleriot_game_get_score (AisleriotGame *game)
 {
-  gint n_cards, exposed;
+  g_return_val_if_fail (AISLERIOT_IS_GAME (game), NULL);
 
-  if (old_cards) {
-    n_cards = (gint) slot->old_cards->len;
-    exposed = (gint) slot->old_exposed;
-  } else {
-    n_cards = (gint) slot->cards->len;
-    exposed = (gint) slot->exposed;
-  }
+  return game->score ? game->score : "";
+}
 
-  if (card_num >= n_cards - exposed) {
-    gint idx = card_num + exposed - n_cards;
-    *xoff = slot->pixeldx * idx;
-    *yoff = slot->pixeldy * idx;
-  } else {
-    *xoff = 0;
-    *yoff = 0;
+static void
+append_games_from_path (GHashTable *hash_table,
+                        const char *path,
+                        const char *suffix)
+{
+  GDir *dir;
+  const char *filename;
+
+  dir = g_dir_open (path, 0, NULL);
+  if (dir == NULL)
+    return;
+
+  while ((filename = g_dir_read_name (dir)) != NULL) {
+    if (!g_str_has_suffix (filename, suffix))
+      continue;
+
+    g_hash_table_insert (hash_table, 
+                         ar_filename_to_game_module (filename),
+                         NULL);
   }
 }
 
-void
-aisleriot_game_reset_old_cards (ArSlot *slot)
+static int
+compare (gconstpointer *a,
+         gconstpointer *b)
 {
-  g_byte_array_set_size (slot->old_cards, 0);
-  g_byte_array_append (slot->old_cards, slot->cards->data, slot->cards->len);
-  slot->old_exposed = slot->exposed;
+  return strcmp ((char *) *a, (char *) *b);
 }
 
-#endif /* HAVE_CLUTTER */
+/**
+ * ar_get_game_modules:
+ * 
+ * Returns: (tranfer full): the list of available games
+ */
+char **
+ar_get_game_modules (void)
+{
+  GHashTable *hash_table;
+  char *path;
+  GPtrArray *array;
+  GHashTableIter iter;
+  gpointer key;
+
+  hash_table = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+
+  path = g_build_filename (ar_runtime_get_directory (AR_RUNTIME_PKG_LIBRARY_DIRECTORY),
+                           "guile",
+                           SCM_EFFECTIVE_VERSION,
+                           NULL);
+  append_games_from_path (hash_table, path, ".go");
+  g_free (path);
+
+  path = g_build_filename (ar_runtime_get_directory (AR_RUNTIME_PKG_DATA_DIRECTORY),
+                           "guile",
+                           SCM_EFFECTIVE_VERSION,
+                           NULL);
+  append_games_from_path (hash_table, path, ".scm");
+  g_free (path);
+
+  array = g_ptr_array_sized_new (g_hash_table_size (hash_table));
+
+  g_hash_table_iter_init (&iter, hash_table);
+  while (g_hash_table_iter_next (&iter, &key, NULL)) {
+    g_hash_table_iter_steal (&iter);
+    g_ptr_array_add (array, key);
+  }
+  g_hash_table_unref (hash_table);
+
+  g_ptr_array_sort (array, (GCompareFunc) compare);
+  g_ptr_array_add (array, NULL);
+
+  return (char **) g_ptr_array_free (array, FALSE);
+}
