@@ -383,21 +383,17 @@ aisleriot_window_show_about_dialog (AisleriotWindow * window)
     /* FIXMEchpe: who did the Bonded cards? */
     /* Tango Nuevo */
     "\"Thom-10\"",
-    /* Tango Nuevo, Rheinland, Ancient French, Simplistic, Anglo Modern,
+    /* Tango Nuevo, Ancient French, Simplistic,
      * Атласные (Atlasnye), Atlasnye (International), Atlasnye Skat,
-     * Tigullio, Anglo Poker, Anglo Modern 4 Color, Plastic, Neoclassical,
-     * Neoclassical 4 Color, Berlin, Lattmann, Orell, Tarocco,
+     * Tigullio, Plastic, Neoclassical, Neoclassical 4 Color,
      * fixes and improvements to other decks.
      */
     "Vincent Bermel",
-    /* Anglo Modern, Anglo Modern 4 Colors,
-     * Атласные (Atlasnye), Atlasnye (International), Atlasnye Skat,
+    /* Атласные (Atlasnye), Atlasnye (International), Atlasnye Skat,
      */
     "Дмитрий Фомин",
     /* Tigullio */
     "Luciano Montanaro",
-    /* Slavic Costumes */
-    "Stanislav Lyashchevsky",
     /* Plastic */
     "Gifford Cheung",
     /* Neoclassical */
